<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class RepositoryApi extends CApplicationComponent
{
    const OBJ_TYPE_FOLDER = 10002;
    const OBJ_TYPE_PAGE = 10007;
    const OBJ_TYPE_BINARY = 10008;

    /**
     * @var string api url
     */
    public $url;

    /**
     * @var array possible content connector failover urls
     */
    public $failover_urls = array();

    /**
     * @var string content connector repository search(ccr) path
     */
    public $contentRepositorySearch;
	
    /**
     * @var string binary content service path
     */
    public $binaryContent;

    /**
     * @var string content connector navigation path
     */
    public $navigation;

    /**
     * @var string content connector activePath link
     */
    public $activePath;

    /**
     * @var string content connector autosuggest path
     */
    public $autosuggest;

    /**
     * @var string content connector search path
     */
    public $search;
	
    /**
     * @var string content connector clear cache path
     */
    public $clearCache;

    /**
     * @var string external host, which can be used as host instead of only relative path
     */
    public $contentExternalHost;

    /**
     * @var array parameters which will be added to every request to gentics content connector
     */
    public $additionalRequestParameters = array();

    /**
     * @var array Attributes, which will be added to every request as "&attribute=something"
     */
    public $additionalAttributes = array();

    /**
     * @var array Sorting-Rules, that can be added to spezific requests
     */
    public $additionalSortingRules = array();

    /**
     * Get api url which used to retrieve CMS objects
     *
     * @return string
     */
    protected function getContentRepositorySearchUrl()
    {
        return $this->getUrl() . $this->contentRepositorySearch;
    }

    /**
     * Get api url which used to retrieve static objects(images, js, css, etc.)
     *
     * @return string
     */
    protected function getBinaryContentUrl()
    {
        return $this->getUrl() . $this->binaryContent;
    }

    /**
     * Get api url which used to retrieve navigation data
     *
     * @return string
     */
    protected function getNavigationUrl()
    {
        return $this->getUrl() . $this->navigation;
    }

    /**
     * Get api url which used to retrieve active path data
     *
     * @return string
     */
    protected function getActivePathUrl()
    {
        return $this->getUrl() . $this->activePath;
    }

    /**
     * Get api url which used to retrieve autosuggest data
     *
     * @return string
     */
    protected function getAutosuggestUrl()
    {
        return $this->getUrl() . $this->autosuggest;
    }

    /**
     * Get api url which used to retrieve search data
     *
     * @return string
     */
    protected function getSearchUrl()
    {
        return $this->getUrl() . $this->search;
    }

    /**
     * Set base options
     */
    public function __construct()
    {
        if (isset($_GET['lang']) && is_string($_GET['lang'])) {
            Yii::app()->language = $_GET['lang'];
        }
    }

    protected function getUrl()
    {
        // The main API was failed and we check if there
        //is detected auxiliary API server in the cache
        if ($failover_URL = Yii::app()->cache->get('failover_URL', false)) {
            $this->failover_urls[] = $this->url;
            $this->url = $failover_URL;
        }
        return $this->url;
    }

    /**
     * Do almost the same as getCmsObject. But in case relative path $path not found in the CMS it adds host address
     * in front of $path and try second time
     *
     * @param $path
     * @param array $attributes
     * @return array|bool
     */
    public function getCmsObjectCanBeExternal($path, $attributes = array())
    {
        $cmsObject = $this->getCmsObject($path, $attributes);
        if (!$cmsObject && !preg_match('/.+\.(php|html)$/', $path)) {
            $externalPath = (!empty($this->contentExternalHost) ? $this->contentExternalHost : Yii::app()->request->getHostInfo()) . $path;
            return $this->getCmsObject($externalPath, $attributes);
        }

        return $cmsObject;
    }

    /**
     * Get CMS content last modification time
     *
     * @param string $path path of node
     *
     * @return bool
     */
    public function getLastModificationTime($path)
    {
        $cacheId = 'updateTMSTMP_'.md5($path);
        $updateTMSTMP = false;
        if (!($updateTMSTMP = Yii::app()->cache->get($cacheId, false))
            || (isset($_GET['refresh']) && $_GET['refresh']=='true')) {
            $attributes = $this->getContentAttributes($path, array('updatetimestamp'));
            if($attributes) {
                $updateTMSTMP = $attributes['updatetimestamp'];
                Yii::app()->cache->set($cacheId, $updateTMSTMP, Yii::app()->getModule('contentSource')->cacheTime);
            }
        }
        return $updateTMSTMP;
    }

    /**
     * This method is used for retrieving only content attributes from Gentics Content Connector
     *
     * @param string $path       content path
     * @param array  $attributes array of personalization attributes
     *
     * @return array
     */
    public function getContentAttributes($path, $attributes)
    {
        $cmsObject = $this->getCmsObject($path, array_filter($attributes, function ($el) {
            return $el != 'content';
        }));
        return $cmsObject['attributes'];
    }

    /**
     * Alias for fetchObjectByPath
     *
     * @param string $path       content path
     * @param array  $attributes requested content attributes
     *
     * @return array|bool
     */
    public function getCmsObject($path, $attributes = array())
    {
        return $this->fetchObjectByPath($path, $attributes);
    }

    /**
     * Get CMS content object
     * If object is binary, make additional request to get binarycontent and put it to 'content' field
     *
     * @param string $path       content path
     * @param array  $attributes requested content attributes
     *
     * @return array|bool
     */
    public function fetchObjectByPath($path, $attributes = array())
    {
        $params = array(
            'filter' => "object.url==\"" . $path . "\"",
            'type' => 'php',
        );

        return $this->fetchObjectByParameters($params, $attributes, $path);
    }

    /**
     * Get a CMS content object by Its content ID
     * If object is binary, make additional request to get binarycontent and put it to 'content' field
     *
     * @param string $contentId  Content ID - e.g.: 10008.123
     * @param array  $attributes requested content attributes
     *
     * @return array|bool
     */
    public function fetchObject($contentId, $attributes = array())
    {
        $params = array(
            'filter' => "object.contentid==\"" . $contentId . "\"",
            'type' => 'php',
        );

        return $this->fetchObjectByParameters($params, $attributes, null);
    }

    /**
     * Get a CMS content object
     * If object is binary, make additional request to get binarycontent and put it to 'content' field
     *
     * @param array $parameters   Parameters to use for finding the object, e.g.: contentid
     * @param array  $attributes  requested content attributes
     *
     * @return array
     */
    public function fetchObjectByParameters($parameters, $attributes = array(), $path)
    {
        if ($path === null) {
            $attributes = array_merge($attributes, array("url"));
        }

        $params = $this->addAdditionalParameters($parameters, 'contentRepositorySearch');
        $response = $this->_request($this->getContentRepositorySearchUrl(), $params, $attributes);
        $cmsObject = self::_prepareResponse($response);

        if (!$cmsObject) {
            return null;
        }

        if ($path === null) {
            $path = $cmsObject['attributes']['url'];
        }

        if (in_array('content', $attributes) && $cmsObject['obj_type'] == self::OBJ_TYPE_BINARY) {
            $cmsObject['attributes']['content'] = $this->getBinaryContent($path);
        }

        return $cmsObject;
    }

    /**
     * Download binary content
     *
     * @param string $path content path
     *
     * @return string binary content
     */
    public function getBinaryContent($path)
    {
        $params = array(
            'filter' => "object.url==\"$path\"",
        );
        $params = $this->addAdditionalParameters($params, 'binaryContent');

        $response = $this->_request($this->getBinaryContentUrl(), $params);
        return $response;
    }

    /**
     * Adds parameters to certain requests
     *
     * @param array  $params array of preset parameters
     * @param string $type   type of request from
     *
     * @return array
     */
    public function addAdditionalParameters($params, $type)
    {
        if (!empty($this->additionalRequestParameters[$type])) {
            foreach ($this->additionalRequestParameters[$type] as $name => $val) {
                $params[$name] = !empty($params[$name]) ? $params[$name] . ' AND ' . $val : $params[$name];
            }
        }
        return $params;
    }

    /**
     * clears the CCR cache
     *
     * @param string $url url of CCR, if not set use configured url
     * @param string $contentid content-ID of page
     *
     * @return array
     */
    public function clearCache($contentid, $url="")
    {
        if (empty($url) || $url="") {
            $url = $this->clearCache;
        }
        // /ccr/clearcache?contentid=10007.123
        $url = $url . '?contentid=' . $contentid;
        /*  do not remove the @ sign below, it is required for exception handling */
        @$content = file_get_contents($url, false);
        
        return $content;
    }
    /**
     * Gets additional attribute by request type
     *
     * @param string $type request type (navigation/content/search)
     *
     * @return string additional attributes
     */
    protected function getAdditionalAttributesByRequestType($type) {

        $attributes = false;
        if (isset($this->additionalAttributes[$type])) {
            $attributes = $this->additionalAttributes[$type];
        }
        if ($attributes && $attributes = explode(',',$attributes)) {
            $attributes = array_map('trim',$attributes);
            $attributes = array_unique($attributes);
            $attributes = '&attributes=' . implode('&attributes=', $attributes);
        }
        return $attributes;
    }
    /**
     * Gets additional attribute by url
     *
     * @param string $url url
     *
     * @return string additional attributes
     */
    protected function getAdditionalAttributesByUrl($url)
    {
        $type = false;
        if (strpos($url, $this->navigation)) {
            $type = 'navigation';
        } elseif(strpos($url, $this->binaryContent)) {
            $type = 'content';
        } elseif(strpos($url, $this->contentRepositorySearch)
            || strpos($url, $this->search)) {
            $type = 'search';
        }
        return $this->getAdditionalAttributesByRequestType($type);
    }

    /**
     * Make request to API
     *
     * @param string $url        url
     * @param array  $params     parameters
     * @param array  $attributes array of Gentics Content Connector attributes which should be fetched
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @throws Exception
     * @return string raw response
     */
    private function _request($url, $params, $attributes = array(), $timeout = "")
    {
        $attr = array();
        array_walk($attributes, function ($item) use (&$attr) {
            $attr[] = "attributes=$item";
        });

        $url = $url . '?' . http_build_query($params);
        foreach($this->additionalSortingRules as $addRuleK => $addRuleV) {
            if( strpos($url,$addRuleK)) {
                $url .= '&sorting='.$addRuleV; 
            }
        }

        /* append the requested attributes, if there are any */
        if (!empty($attr)) {
            $url .= '&' . implode('&', $attr);
        }

        /* adding additional attributes depends on request type */
        $url .= $this->getAdditionalAttributesByUrl($url);

        Yii::trace($url, 'Api requests(start)');
        $start = microtime(true);
        
        if (!isset( $timeout ) || $timeout == "") {
            $ctx=stream_context_create(array('http'=>
                array(
                    'timeout' => 60 // 1 minute - if request is longer than go to failover
                )
            ));
        }

        /*  do not remove the @ sign below, it is required for exception handling */
        @$content = file_get_contents($url, false, $ctx);

        $failover_attempt = 0;

        while (($content == false || strpos($content,'DatasourceException')!==false) && ($failover_attempt < count($this->failover_urls))) {
            if (isset( $this->failover_urls[$failover_attempt])) {
                $this->url = $this->failover_urls[$failover_attempt];
                $exploded_url = explode('://',$url);
                $exploded_url = explode('/',$exploded_url[1]);
                $exploded_url[0] = $this->url;
                $url = implode('/', $exploded_url);
                @$content = file_get_contents($url, false, $ctx);
            }
            
            if ($content !== false) {
                Yii::app()->cache->set('failover_URL', $this->url, Yii::app()->getModule('contentSource')->cacheTime);
                break;
            }
            $failover_attempt++;
        }
        if ($content == false) {
            throw new Exception('Repository API error: could not complete API request.');
        }
        Yii::trace($url, 'Api requests(end). Time s.: ' . round(microtime(true) - $start, 4));

        return $content;
    }

    /**
     * Requesting for content object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @return string
     */
    public function request($params, $attributes = array(), $timeout = "")
    {
        return $this->_request($this->getContentRepositorySearchUrl(), $params, $attributes, $timeout);
    }

    /**
     * Requesting for navigation object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @return string
     */
    public function requestNavigation($params, $attributes = array(), $timeout = "")
    {
        return $this->_request($this->getNavigationUrl(), $params, $attributes, $timeout);
    }

    /**
     * Requesting for active path object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @return string
     */
    public function requestActivePath($params, $attributes = array(), $timeout = "")
    {
        return $this->_request($this->getActivePathUrl(), $params, $attributes, $timeout);
    }

    /**
     * Requesting for autosuggest data
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @return string
     */
    public function requestAutosuggest($params, $attributes = array(), $timeout = "")
    {
        return $this->_request($this->getAutosuggestUrl(), $params, $attributes, $timeout);
    }

    /**
     * Requesting for search data
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     * @param string $timeout specific request timeout in seconds (if not set standard timeout 60 seconds is used)
     *
     * @return string
     */
    public function requestSearch($params, $attributes = array(), $timeout = "")
    {
        return $this->_request($this->getSearchUrl(), $params, $attributes, $timeout);
    }

    /**
     * Prepare API response. Convert response to useful form.
     *
     * @param array $response prepared response
     *
     * @return array|bool prepared object.
     *
     */
    private static function _prepareResponse($response)
    {
        $responseData = unserialize($response);
        if (@$responseData['status'] == 'ok') {
            unset($responseData['status']);
            if (count($responseData) > 0) {
                return array_shift($responseData);
            }
            return false;
        } else {
            Yii::log(print_r($responseData, true), CLogger::LEVEL_TRACE);
        }
        return false;
    }
}