<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget, which renders the Marketing Automation tracking code
 */
class MarkaTrackingWidget extends BaseJuiWidget
{
    /**
     * The ID of the page that should be tracked. This is not the CMS page ID,
     * but an ID that is only used for Marketing Automation. It must be a string
     * of the following format:
     * for pages without a channel ID: CMSPageID (e.g. '1234' or '4923')
     * for pages with a channel ID: 'c' . ChannelID . '.' . CMSPageID (e.g. 'c2.1234' or 'c8.4923')
     * @var string 
     */
    public $pageId;
    
    /**
     * The array (map) of the attributes of the page that should be tracked.
     * The array must map each attribute name to a number, which is greater
     * than or equal to 1.0.
     * For convenience the widget will not produce an error if some (but not all)
     * of the attributes are invalid. Attributes with empty names or non numeric values 
     * are removed and if an attribute name contains the '.' character, that 
     * character is replaced with an '_' character and if an attribute value is
     * less than 1.0, it is replaced with the value 1. The '.' restriction is 
     * because Marketing Automation does not support the '.' character, due to a
     * restriction of ElasticSearch and values less than 1.0 are disallowed, 
     * because the Marketing Automation clustering algorithm requires values
     * greater than or equal to 1.0.
     * Valid example: array('attr1' => 1.0, 'attr2' => 1.0)
     * @var array 
     */
    public $attributes;
    
    /**
     * The optional array of the categories of the page that should be tracked.
     * This is optional and if set, it must be a list of strings, e.g. array('category1', 'category2')
     * @var array 
     */
    public $categories = NULL;
    
    /**
     * The method that should be used for tracking. This can be either
     * 'GET' or 'POST'. If 'GET' is specified, a tracking pixel will be rendered
     * (an image, which has all tracking information embedded in its src URL) 
     * and if 'POST' is specified, a POST request will be made using AJAX.
     * If this parameter is not specified, 'GET' is used.
     * @var string 
     */
    public $method = 'GET';
    
    /**
     * Renders the view.
     * 
     * @return void
     */
    public function run()
    {
        if (empty($this->pageId)) {
            die('No pageId specified.');
        }
        if (empty($this->attributes)) {
            die('No attributes specified.');
        }
        $this->_checkAttributes();
        $markaModule = Yii::app()->getModule('marketingautomation');
        $viewParameters = array('trackerBaseUrl' => $markaModule->getMarkaTrackerBaseUrl(),
            'userId' => $markaModule->getUUID(), 'pageId' => $this->pageId, 
            'attributes' => $this->attributes, 'categories' => $this->categories);
        $methodLowerCase = strtolower($this->method);
        switch ($methodLowerCase) {
        case 'get':
            $this->render('MarkaTrackingWidget_get', $viewParameters);
            break;
        case 'post':
            $this->render('MarkaTrackingWidget_post', $viewParameters);
            break;
        default:
            die('Invalid method specified. The method must be either GET or POST.');
        }
    }
    
    /**
     * Checks the attributes the user specified for validity.
     * If an attribute name is empty, it is removed, if an attribute name
     * contains the '.' chracter, that character is replaced with the '_' character.
     * If an attribute value is not numeric, it is removed, if an attribute
     * value is less than 1.0, it is set to 1.
     * When this function returns, the attributes property of this object
     * has been set to a new array, which contains only the valid attributes.
     * If there are no valid attributes, an error is produced.
     */
    private function _checkAttributes()
    {
        $newAttributes = array();
        foreach ($this->attributes as $key => $value) {
            if (is_string($key)) {
                $keyStr = $key;
            } else {
                $keyStr = strval($key);
            }
            if ($keyStr == '') {
                continue;
            }
            if (strpos($keyStr, '.') !== false) {
                $keyStr = str_replace('.', '_', $keyStr);
            }
            
            if (!is_numeric($value)) {
                continue;
            }
            if ($value >= 1.0) {
                $valueToSet = $value;
            } else {
                $valueToSet = 1;
            }
            $newAttributes[$keyStr] = $valueToSet;
        }
        $this->attributes = $newAttributes;
        if (empty($this->attributes)) {
            die('All specified attributes are invalid.');
        }
    }
}
