<?php
/**
* Gentics Portal.Node PHP
* Author & Copyright (c) by Gentics Software GmbH
* sales@gentics.com
* http://www.gentics.com
* Licenses can be found in the LICENSE.txt file in the root-folder of this installation
* You must not use this software without a valid license agreement.
*
* ImportFile model class file of uploaded file .
 *
 * @property string $file
 * @property string $hashname
 * @property string $fieldNames
 * @property string $rowsCount
 * @property string $hasErrors
 * @property object $csv_file
 * @property boolean $valid
*/
class ImportFile extends CFormModel {
    /**
     * @var string real file name in a user filesystem
     */
    public $file;
    /**
     * @var string hash name of the file after uploading
     */
    public $hashname;
    /**
     * @var string names of columns in the uploaded file
     */
    protected $fieldNames;
    /**
     * @var string count of rows if the uploaded file
     */
    protected $rowsCount;
    /**
     * @var boolean shows if there is validation errors in the file
     */
    protected $hasErrors = false;
    /**
     * @var object of csv file
     */
    protected $csv_file = null;
    /**
     * @var boolean shows if there is validation errors in the file
     */
    public $valid = false;
    /**
     * Returns the array of rules
     * @return array rules
     */
    public function rules(){
        return array(
            array('file', 'required'),
            array('file', 'file', 'types'=>'csv'),
        );
    }
    /**
     * Returns the array of the column names
     * @return array column names
     */
    public function getFieldNames(){
        if(!isset($this->fieldNames)){
            $csv_users = new CSVFile($this->getPathToFile());
            $csv_users->rewind();
            $this->fieldNames = $csv_users->getKeys();
        }
        return $this->fieldNames;
    }
    /**
     * Returns the flag if there is validation errors
     * @return boolean flag if there is errors in the file
     */
    public function hasErrors(){
        return $this->hasErrors;
    }
    /**
     * Set there is an error in the file
     */
    public function assignError(){
        $this->hasErrors = true;
    }
    /**
     * Returns the number of rows in the file
     * @return integer the number of records
     */
    public function getRowsCount(){
        if(!isset($this->rowsCount) && $this->isExist()){
            $csv_users = new CSVFile($this->getPathToFile());
            $csv_users->seek($csv_users->getSize());
            $linesTotal = $csv_users->key();
            $this->rowsCount = $linesTotal - 1;
        }elseif(!$this->isExist()){
            $this->rowsCount = 0;
        }
        return $this->rowsCount;
    }
    /**
     * Returns the path to directory with uploaded files
     * @return string path to uploaded files
     */
    public static function getPathToFiles(){
        return Yii::getPathOfAlias('user.CSVImportFiles');
    }
    /**
     * Returns the path to the uploaded file
     * @return string path to uploaded file
     */
    public function getPathToFile(){
        return isset($this->hashname)?(Yii::getPathOfAlias('user.CSVImportFiles').'/'.$this->hashname):false;
    }
    /**
     * Returns the flag if uploaded file exists
     * @return boolean flag the uploaded file exists
     */
    public function isExist(){
        $pathToFile = $this->getPathToFile();
        return $pathToFile? file_exists($pathToFile) : false;
    }
    /**
     * Returns the model of necessary class created with values from the passed row
     * @param object $model active record model instance
     * @param array $csv_row an array with form fieldName=>values
     * @return object the model with filled attributes
     */
    public function getFromCsvRow($model,$csv_row){
        $intersect_to_model = array_intersect_key($csv_row, $model->attributes);
        if(empty($intersect_to_model)){
            return false;
        }
        $model->attributes  =  $intersect_to_model;

        return $model;
    }
    /**
     * Returns the object of csv file
     * @return object the instance of CSVFile class
     */
    public function getCsvFile(){
        if($this->isExist()){
            if(!isset($this->csv_file)){
                $this->csv_file = new CSVFile($this->getPathToFile());
            }
            return $this->csv_file;
        }else{
            return array();
        }
    }
    /**
     * Method looking for the same records in database and uses them if them are existed
     * @param object $user a user model instance
     * @param object $profile a profile model instance
     */
    public static function overrideExisted(&$user,&$profile){

        $tmpUser = User::model()->find('email=:email',array('email'=>$user->email));
        if($tmpUser){
            foreach($user->attributes as $k => $attribute){
                if(isset($attribute) && !empty($attribute)){
                    $tmpUser->setAttribute($k, $attribute);
                }
            }
            $user = clone $tmpUser;
            if(isset($scenarioNotUniq)){
                $user->setScenario($scenarioNotUniq);
            }
        }
        if($profile){
            $tmpProfile = Profile::model()->findByPk($user->id);
            if($tmpProfile){
                foreach($profile->attributes as $k => $attribute){
                    if(isset($attribute) && !empty($attribute)){
                        $tmpProfile->setAttribute($k, $attribute);
                    }
                }
                $profile = clone $tmpProfile;
            }
        }
    }
    /**
     * Returns an array comprising names of profiles fields
     * @return array the names of profiles fields
     */
    public function getProfileFieldNames(){
        $profile = new Profile;
        $profilesFieldNames = array_intersect($this->getFieldNames(),array_keys($profile->attributes));
        unset($profile);
        return $profilesFieldNames;
    }
    /**
     * Returns an array comprising names of users fields
     * @return array the names of users fields
     */
    public function getUserFieldNames(){
        $user = new User;
        $userFieldNames = array_intersect($this->getFieldNames(),array_keys($user->attributes));
        unset($user);
        return $userFieldNames;
    }
    /**
     * Returns the model of user
     * @param array $csv_row an array with form fieldName=>values
     * @return object the model with filled attributes
     */
    public function getUserFromCsvRow($csv_row){
        return self::getFromCsvRow(new User('import'), $csv_row);
    }
    /**
     * Returns the model of profile
     * @param array $csv_row an array with form fieldName=>values
     * @return object the model with filled attributes
     */
    public function getProfileFromCsvRow($csv_row){
        return self::getFromCsvRow(new Profile('import'), $csv_row);
    }
}